<?php
/*
 * CCViews version 1.1.10
 * A parameterizable nested view loader with a global-scope function API.
 * 
 * Copyright 2008-2009, Peter Rowntree.
 * http://www.hdyn.com/mail/contact.php?addr=pr
 *
 * License: you may not use this file except in compliance with
 * the Apache License, Version 2.0, a copy of which you may obtain at
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * needs PHP 5.
 * URL syntax: holder.php?d0=a,b;c,d&d1=p,q,r;s,t...&dn=x,y,x&trace
 */

//-----editing these values does not constitute a code modification-------------
$g_alwaysDoTrace=false;	//=>trace even when 'trace' is not set in the query string
//------------------------------------------------------------------------------
	
class CCViews
{
	public $viewFileExtension;
	protected $maxDepth;
	protected $depth=0;
	protected $nViewsLoaded=0;
	protected $depthIView=array();	//numeric; current view index at each depth
	protected $views=array();	//3-d numeric; [level][group index]=>numeric array of view names
	protected $map=array();	//to hold any variables used by the views
	protected $traceTree=array();
	protected $doTrace=false;

	function __construct($maxDepth=10,$viewFileExtension=".thtml")
	{
		global $g_alwaysDoTrace;
		$this->maxDepth=$maxDepth;
		$this->viewFileExtension=$viewFileExtension;
		$this->doTrace = $g_alwaysDoTrace || isset($_GET["trace"]);
	}

	/**
	 *
	 * @param array $inArr usually $_GET or $_POST;
	 */
	function setViews(&$inArr)
	{
		for($i=0; $i<$this->maxDepth; $i++)
		{
			$key="d".$i;
			if(isset($inArr[$key]))
			{
				$depthViews=preg_split("/\\s*;\\s*/s",$inArr[$key]);
				foreach($depthViews as $j=>$group)
				{
					$newGroupArr=preg_split("/\\s*,\\s*/s",$group);
					if(isset($this->views[$i][$j]))	//replace only when new is non-empty
						self::nneMerge($this->views[$i][$j],$newGroupArr);
					else
						$this->views[$i][$j]=$newGroupArr;
				}
			}
		}
		return $this;
	}
	
	/**
	 * "numeric non-empty merge"
	 * a 'merge' function for numeric arrays that doesn't just append as array_merge does, 
	 * with the additional constraint that the new value is only merged if it is not "".
	 *
	 * @param array ref $arr
	 * @param array ref $newArr
	 */
	static function nneMerge(&$arr,&$newArr)
	{
		$n=count($arr);
		foreach($newArr as $i=>$val)
		{
			if($val == "")
				continue;
			if($i >= $n)
				$arr[]=$val;	//append
			else
				$arr[$i]=$val;	//replace
		}
	}
	
	function gotView($viewName)
	{
		foreach($this->views as $depth)
		{
			foreach($depth as $group)
			{
				if(in_array($viewName,$group))
					return true;
			}
		}
		return false;
	}
	
	function viewAt($depth,$iGroup,$i)
	{
		if(isset($this->views[$depth][$iGroup][$i]))
			return $this->views[$depth][$iGroup][$i];
		return false;
	}

	function get($k)
	{
		return $this->map[$k];
	}

	function set($k,$v)
	{
		$this->map[$k]=$v;
	}
	
	/**
	 * this method loads all the views, so:
	 * 	in the top-level scope of views, $this is the CCViews object;
	 * 	globals need to be set using setGlobal or setGlobals
	 *
	 * @param string $viewName
	 * @return true if the view has children
	 */
	function loadView($viewName)
	{
		$path=$this->getViewPath($viewName);
		if(!file_exists($path))
			return;
		$nLoaded=++$this->nViewsLoaded;
		include $path;
		return $this->nViewsLoaded > $nLoaded;
	}
	
	function getViewPath($viewName)
	{
		return dirname(__FILE__).DIRECTORY_SEPARATOR."views"
			.DIRECTORY_SEPARATOR.$viewName.$this->viewFileExtension;
	}
	
	/**
	 * get a view by index at the current depth
	 *
	 * @param int $i the index
	 */
	function viewByIndex($i)
	{
		if(!isset($this->depthIView[$this->depth]))
			$this->depthIView[$this->depth]=0;
		$tracePrefix=$this->depthIView[$this->depth]."[".$i."]:";
		if(isset($this->views[$this->depth][$this->depthIView[$this->depth]][$i]))
			$this->viewByName($this->views[$this->depth][$this->depthIView[$this->depth]][$i],$tracePrefix);
		else
			$this->trace($tracePrefix."-");
		return $this;
	}
	
	function viewByName($viewName,$tracePrefix="name:")
	{
		$this->trace($tracePrefix.$viewName);
		if($viewName === "")
			return;
		++$this->depth;
		if(!isset($this->depthIView[$this->depth]))
			$this->depthIView[$this->depth]=0;
		if($this->depth <= $this->maxDepth
			&& $this->loadView($viewName))	//view has children
			++$this->depthIView[$this->depth];
		--$this->depth;
		return $this;
	}
	
	function trace($str)
	{
		if($this->doTrace)
			$this->traceTree[]=str_repeat("|",$this->depth).htmlspecialchars($str);
	}
	
	function showTrace()
	{
		if($this->doTrace)
			echo "<div class='trace'><p>----trace----<br/>&nbsp;</p><p>",implode("</p><p>",$this->traceTree),"</p></div>";
	}
	
	/**
	 * sets key/value pairs in a target array from a defaults array iff they do 
	 * not already exist in the target array
	 * PR~090424: replaced by nneMerge
	 *
	 * @param array $defArr	the defaults array
	 * @param array $tgtArr the target array
	 */
	static function setKVArrDefaults(&$defArr,&$tgtArr)
	{
		$tgtArr=array_merge($defArr,$tgtArr);
	}
	
	/**
	 * Creates a CCViews object
	 *
	 * @param array $defaultsKVArr an assoc array specifying defaults for unspecified $_GET's
	 * @param int $maxDepth
	 * @param string $viewFileExtension
	 * @return CCViews object $g_viewsOb
	 */
	static function create($defaultsKVArr=null,$maxDepth=10,$viewFileExtension=".thtml")
	{
		$viewsOb=new CCViews($maxDepth,$viewFileExtension);
		if(is_array($defaultsKVArr))
			$viewsOb->setViews($defaultsKVArr);
		$viewsOb->setViews($_POST)->setViews($_GET);
		return $viewsOb;
	}
	
}	//end CCViews

//--------------------------helpers---------------------------------------------

function setGlobal($k,$v)
{
	$GLOBALS[$k]=$v;
}

function setGlobals($gArr)
{
	foreach($gArr as $k=>$v)
		$GLOBALS[$k]=$v;
}

function setCSS()
{
	echo "<style type='text/css'>";
	foreach(func_get_args() as $x)
		echo "@import url(",$x,");";
	echo "</style>";
}

?>