<?php
/**
 * DBAccess
 * a general-purpose MySQL connector with a global-scope function API.
 * 
 * Copyright 2006-2008, Peter Rowntree.
 * http://www.hdyn.com/mail/contact.php?addr=pr
 *
 * License: you may not use this file except in compliance with
 * the Apache License, Version 2.0, a copy of which you may obtain at
 *   http://www.apache.org/licenses/LICENSE-2.0
 * 
 * needs PHP 5.
 * Doesn't backtick table and field names.
 * Unless get_magic_quotes_gpc() is true, slashes insert and update values.
 * Encloses insert and update values in double-quotes.
 */
	
class DBAccess
{
	protected $dblink=false;
	//-1:print to screen; 0:system logger; 1:email to logDestination; 3:append to file logDestination
	protected $logType;
	protected $logDestination;	//file path or email address
	
	function __construct($logType=-1,$logDestination=null)
	{
		if($logType > 0 && $logDestination == null)	//not fatal; leave defaults
			$this->gotErr(true,"log type > 0 requires a destination.");
		else
		{
			$this->logType=$logType;
			$this->logDestination=$logDestination;
		}
	}
	
	function connect($host,$user,$passwd,$dbName=null)
	{
		$this->dblink=@mysql_connect($host,$user,$passwd);
		if($this->gotSQLErr($this->dblink,"connect error"))
			return false;
		if($dbName !== null && !$this->useDB($dbName))
			return false;
		return true;
	}

	function useDB($dbName)
	{
		return !$this->gotSQLErr(@mysql_query("USE $dbName",$this->dblink),"useDB error");
	}

	function getRecords($tblName,$fldNames=null,$cond=null,$more=null)
	{
		$query=buildSelect($tblName,$fldNames);
		if($cond != null)
			$query.=" WHERE ".$cond;
		if($more != null)
			$query.=" ".$more;
		$result=@mysql_query($query,$this->dblink);
		if($this->gotSQLErr($result,"getRecords error"))
			return false;
		return $result;
	}

	//returns last insert ID
	//For LAST_INSERT_ID(), the most recently generated ID is maintained in the server
	//on a per-connection basis. It is not changed by another client. It is not even
	//changed if you update another AUTO_INCREMENT  column with a non-magic value
	//(that is, a value that is not NULL and not 0). Using LAST_INSERT_ID() and
	//AUTO_INCREMENT columns simultaneously from multiple clients is perfectly valid.
	//Each client will receive the last inserted ID for the last statement that client
	//executed.
	function insertRecord($tblName,$fldNames,$fldVals)
	{
		if($this->gotSQLErr(@mysql_query(buildInsert($tblName,$fldNames,$fldVals),$this->dblink),"insertRecord error"))
			return false;
		$result=@mysql_query("SELECT LAST_INSERT_ID() AS lastID",$this->dblink);
		if($this->gotSQLErr($result,"select last insert ID error"))
			return false;
		$result=@mysql_fetch_row($result);
		if($result === false || !is_array($result) || count($result) < 1)
			return false;
		return $result[0];
	}

	function deleteRecords($tblName,$cond)
	{
		return !$this->gotSQLErr(@mysql_query(buildDelete($tblName,$cond),$this->dblink),"deleteRecords error");
	}
	
	function updateRecords($tblName,$fldNames,$fldVals,$cond)
	{
		return !$this->gotSQLErr(@mysql_query(buildUpdate($tblName,$fldNames,$fldVals,$cond),$this->dblink),"updateRecords error");
	}

	function rawQuery($query)
	{
		$result=@mysql_query($query,$this->dblink);
		if($this->gotSQLErr($result,"rawQuery error"))
			return false;
		return $result;
	}

	function gotSQLErr($result,$errMsg)
	{
		return $this->gotErr($result === false,$errMsg.": ".mysql_error());
	}

	function gotErr($errCond,$errMsg)
	{
		if(!$errCond)
			return false;
		if($errMsg != null)
		{
			switch($this->logType)
			{
				case 0:	//to system
					@error_log(buildInfoLines("\n",true).$errMsg,$this->logType);
					break;
				case 3:	//to file
					@error_log(buildInfoLines("\n",true).$errMsg,$this->logType,$this->logDestination);
					break;
				case 1:
					$headers  = "MIME-Version: 1.0\n";
					$headers .= "Content-type: text/html; charset=iso-8859-1\n";
					$headers .= "From: {$this->logDestination}\n";
					@error_log(buildInfoLines().htmlspecialchars($errMsg),1,$this->logDestination,$headers);
					break;
				default:
					throw new Exception($errMsg);
					break;
			}
		}
		return true;
	}
}	//end DBAccess

//--- helpers ------------------------------------------------------------------

function slash(&$fldVals)
{
	if(get_magic_quotes_gpc())
		return;
	$length=count($fldVals);
	for($i=0; $i<$length; $i++)
		$fldVals[$i]=addslashes($fldVals[$i]);
}

function buildInfoLines($lf="<br/>",$omitDate=false)
{
	$arr=array();
	if(!$omitDate)
		$arr[]=date('ymd.His T');
	$arr[]="REMOTE_ADDR=".$_SERVER['REMOTE_ADDR'];
	$arr[]="HTTP_USER_AGENT=".$_SERVER['HTTP_USER_AGENT'];
	return $lf.implode($lf,$arr).$lf;
}

//$fldNames == null => all (*)
function buildSelect($tblName,$fldNames=null)
{
	$fldsStr=$fldNames !== null && count($fldNames) > 0 ? implode(",",$fldNames) : "*";
	return "SELECT ".$fldsStr." FROM ".$tblName;
}

function buildInsert($tblName,$fldNames,$fldVals)
{
	slash($fldVals);
	if($fldNames === null || count($fldNames) < 1)
		return "INSERT INTO $tblName VALUES()";
	return "INSERT INTO $tblName (".implode(",",$fldNames).") VALUES(\"".implode("\",\"",$fldVals)."\")";
}

function buildDelete($tblName,$cond)
{
	return "DELETE FROM $tblName WHERE $cond";
}

//expects $fldNames & $fldVals to be correct
function buildUpdate($tblName,$fldNames,$fldVals,$cond)
{
	$length=count($fldNames);
	slash($fldVals);
	$arr=array();
	for($i=0; $i<$length; $i++)
		$arr[]=$fldNames[$i]."=\"".$fldVals[$i]."\"";
	$query="UPDATE $tblName SET ".implode(",",$arr);
	if($cond !== null)
		$query.=" WHERE ".$cond;
	return $query;
}

function rsToArray($rs,$fetchNumeric=false)
{
	if(is_bool($rs))
		return $rs;
	$rows=array();
	if($fetchNumeric)
	{
		while(($row=@mysql_fetch_row($rs)) !== false)
			$rows[]=$row;
	}
	else
	{
		while(($row=@mysql_fetch_assoc($rs)) !== false)
			$rows[]=$row;
	}
	return $rows;
}

//-------------a global-scope function API--------------------------------------

$g_dba=null;
function createGlobalDBA($connect=true)
{
	global $g_dba;
	
	$g_dba=new DBAccess();	//[$logType][,$logDestination]
	if($connect && !$g_dba->connect("localhost","","",""))	//$host,$user,$passwd[,$dbName]
		throw new Exception("DBAccess failed to connect.");
	return $g_dba;
}

//-------- standard CRUD; all return false on error---------------

//returns last insert ID
function create($tbl,$keyArr,$valArr)
{
	global $g_dba;
	return $g_dba->insertRecord($tbl,$keyArr,$valArr);
}

//now, always returns rs; expects caller to call rsToArray
//to convert rs to an array.
function read($tbl,$keyArr=null,$cond=null,$more=null)
{
	global $g_dba;
	return $g_dba->getRecords($tbl,$keyArr,$cond,$more);
}

function update($tbl,$keyArr,$valArr,$cond=null)
{
	global $g_dba;
	return $g_dba->updateRecords($tbl,$keyArr,$valArr,$cond);
}

function delete($tbl,$cond)
{
	global $g_dba;
	return $g_dba->deleteRecords($tbl,$cond);
}

//--------------------------------------------------
/**
 * 
 * @param string $tbl table name
 * @param string $cond condition
 * @param string $fld a field name (usually the primary key) if a field name is known, else uses "*"
 * @return boolean false on error
 * /
function exists($tbl,$cond=null,$fld=null)
{
	global $g_dba;
	$rs=$g_dba->getRecords($tbl,$fld === null ? null : array($fld),$cond,"LIMIT 1");
	if($rs === false)
		return false;   //error already logged
	return @mysql_num_rows($rs) > 0;
}

function rawQuery($query)
{
	global $g_dba;
	return $g_dba->rawQuery($query);
}

?>